﻿init python:
    from copy import copy
    import math
    import random
    import pygame
    import time
    random.seed(time.perf_counter_ns())

    def get_angle(position, destination):
        dx = destination[0] - position[0]
        dy = destination[1] - position[1]
        theta = math.atan2(dy, dx)
        return theta
    def distance(a, b):
        if a >= b:
            result = a - b
        else:
            result = b - a
        return result

    class rps_unit(renpy.Displayable):
        def __init__(self, index, name, image):
            super(renpy.Displayable,self).__init__()
            self.index = index
            self.name = name
            self.image = image

            self.health = 10
            self.team = None
            self.parent = None
            self.flip = 1
            self.speed = 2
            self.visible = True
            self.x = 0
            self.y = 0
            self.target = None
        def find_a_target(self):
            if self.target:
                if self.target.visible:
                    return
            pool = self.parent.fighters.copy()
            random.shuffle(pool)
            for i in pool:
                if i.visible:
                    if i.team != self.team:
                        self.target = i
                        return
            self.target = None
        def knock_back(self, damage):
            if self.flip > 0:
                self.x += 40
            else:
                self.x -= 40
            self.y += random.choice([-20, 20])
            self.health -= damage
            if self.health < 1:
                self.visible = False
                self.parent.winner_check()
        def render(self,width,height,st,at):
            if not self.visible:
                return renpy.Render(0, 0)
            
            self.find_a_target()
            if self.target:
                if self.x > self.target.x:
                    self.flip = 1
                else:
                    self.flip = -1
                for i in self.parent.fighters:
                    if i == self:
                        break
                    if i.team == self.team:
                        break
                    if i.visible:
                        if distance(i.x, self.x) < 20:
                            if distance(i.y, self.y) < 20:
                                if i.index + 1 < self.index:
                                    i.knock_back(5)
                                    self.knock_back(1)
                                elif i.index - 1 > self.index:
                                    i.knock_back(1)
                                    self.knock_back(5)
                                elif i.index == 0 and self.index == 4:
                                    i.knock_back(5)
                                    self.knock_back(1)
                                else:
                                    i.knock_back(1)
                                    self.knock_back(1)
                                break

                angle = get_angle((self.x,self.y), (self.target.x,self.target.y))
                xVelocity = self.speed * math.cos(angle)
                yVelocity = self.speed * math.sin(angle)
                self.x += xVelocity
                self.y += yVelocity
                if self.x > 900:
                    self.x = 900
                elif self.x < -900:
                    self.x = -900
                if self.y > 500:
                    self.y = 500
                elif self.y < -500:
                    self.y = -500

            img = self.image + " " + self.team
            t = Transform(img, xanchor=0.5, yanchor=0.5, zoom = 4, xzoom = self.flip, nearest = True)
            child_render = renpy.render(t, width, height, st, at)
            cw, ch = child_render.get_size()

            rv = renpy.Render(cw, ch)
            rv.blit(child_render, (self.x, self.y))

            renpy.redraw(self, 0)
            return rv

    class rps_team:
        def __init__(self):
            self.team = []
            self.points = 15
            self.team_count = 0
            self.is_player = False
        def count_team(self):
            count = 0
            for i in self.team:
                count += i[0]
            self.team_count = count

        def remove(self, monster):
            for i in self.team:
                if i[1].name == monster.name:
                    if i[0] > 1:
                        i[0] -= 1
                    else:
                        self.team.remove(i)
                    break
            self.count_team()
        def add(self, monster):
            if self.team_count >= self.points:
                return
            for i in self.team:
                if i[1].name == monster.name:
                    i[0] += 1
                    break
            else:
                if len(self.team) < 3:
                    self.team.append([1, monster])
            self.count_team()
        def generate_monsters(self):
            monsters = []
            for j in self.team:
                for i in range(j[0]):
                    fighter = copy(j[1])
                    monsters.append(fighter)
            return monsters
    class rps_manager:
        def __init__(self, creatures, team_a, team_b):
            self.creatures = creatures
            self.fighters = []
            self.team_a = team_a
            self.team_b = team_b
            self.team_a_points = 15
            self.team_b_points = 15
            self.winner = None
        def winner_check(self):
            a = 0
            b = 0
            for i in self.fighters:
                if i.visible:
                    if i.team == "a":
                        a += 1
                    else:
                        b += 1
            if a == 0 and b == 0:
                self.winner = "It's a draw."
            elif a == 0:
                self.winner = "Team B have won."
            elif b == 0:
                self.winner = "Team A have won."
            self.team_a_points = a
            self.team_b_points = b
            renpy.restart_interaction()
        def build_team_a(self):
            team_count = 0
            for i in self.team_a.team:
                team_count += i[0]
            if team_count >= self.team_a.points:
                return
            monster = random.choice(self.creatures)
            self.team_a.add(monster)
            self.build_team_a()

        def fill_teams(self):
            self.winner = None
            self.team_a_points = 15
            self.team_b_points = 15
            self.fighters = []
            self.place_team_a()
            self.place_team_b()
        def place_team_a(self):
            for fighter in self.team_a.generate_monsters():
                fighter.team = "a"
                fighter.parent = self
                fighter.x = random.randint(-900, -300)
                fighter.y = random.randint(-500, 500)
                fighter.speed = 1 + random.random()
                self.fighters.append(fighter)
        def place_team_b(self):
            for fighter in self.team_b.generate_monsters():
                fighter.team = "b"
                fighter.parent = self
                fighter.x = random.randint(300, 900)
                fighter.y = random.randint(-500, 500)
                fighter.speed = 1 + random.random()
                self.fighters.append(fighter)

image rps_dragon a = Crop((0,0,16,16), "rps rush/sprites.png")
image rps_phenix a = Crop((16,0,16,16), "rps rush/sprites.png")
image rps_ghost a = Crop((32,0,16,16), "rps rush/sprites.png")
image rps_knight a = Crop((48,0,16,16), "rps rush/sprites.png")
image rps_worm a = Crop((0,16,16,16), "rps rush/sprites.png")

image rps_dragon b = Crop((0,32,16,16), "rps rush/sprites.png")
image rps_phenix b = Crop((16,32,16,16), "rps rush/sprites.png")
image rps_ghost b = Crop((32,32,16,16), "rps rush/sprites.png")
image rps_knight b = Crop((48,32,16,16), "rps rush/sprites.png")
image rps_worm b = Crop((0,48,16,16), "rps rush/sprites.png")

default rps_1 = rps_manager(
        creatures = [
            rps_unit(0, "Dragon", "rps_dragon"),
            rps_unit(1, "Phenix", "rps_phenix"),
            rps_unit(2, "Ghost", "rps_ghost"),
            rps_unit(3, "Knight", "rps_knight"),
            rps_unit(4, "Worm", "rps_worm"),
        ],
        team_a = rps_team(),
        team_b = rps_team(),
    )

screen rps(g = rps_1):
    modal True
    style_prefix "rps"
    for i in g.fighters:
        add i align .5,.5
    hbox:
        align .5,1.0 spacing 10 yoffset -20
        button:
            text "Restart"
            action Function(g.fill_teams)
        button:
            text "Return"
            action Rollback()

    text str(g.team_a_points) align .0,.0 offset 20,20
    text str(g.team_b_points) align 1.0,.0 offset -20,20

    if g.winner:
        frame:
            padding 20,20 background "#220039"
            vbox:
                spacing 10
                text g.winner align .5,.5
                button:
                    text "Done"
                    action Return()


screen rps_choose(g = rps_1, points = 15):
    modal True
    style_prefix "rps"
    frame:
        align .5,.5
        vbox spacing 10:
            text "Choose your team"
            hbox spacing 10:
                for i in g.creatures:
                    vbox:
                        text i.name xalign .5
                        button:
                            add i.image + " b"  at zoom(8, True)
                            action Function(g.team_b.add, i)
            text "You have [g.team_b.team_count] / [g.team_b.points]."
            hbox spacing 10:
                align .5,.5
                for i in g.team_b.team:
                    vbox:
                        button:
                            add i[1].image + " b" at zoom(8, True)
                            action Function(g.team_b.remove, i[1])
                        text str(i[0])
            button:
                text "Done"
                action Return()

style rps_button:
    background "#310039" padding (10,10) align (.5,.5)
style rps_frame:
    background "#00741199" padding (10,10) align (.5,.5)
style rps_text:
    align (.5,.5)



label rps_example:
    show image "#0c0014" with dissolve
    call screen rps_choose()
    $ rps_1.build_team_a()
    $ rps_1.fill_teams()
    call screen rps

